c
      program fftfault
c
c******************************************************************
c  REFERENCE:
c  Smith, B. and D. Sandwell, Coulomb Stress Accumulation Along the 
c  San Andreas Fault System, J. Geophys. Res., in press, 2003.
c******************************************************************
c
c  USAGE:
c  Program to calculate the surface displacement or stress
c  due to planar vertical fault consisting of many elements
c  where the slip varies on each element.
c  All elements slip between depths D1 and D2. 
c  D1 can be set to a great depth (e.g., -100000m) to simulate
c  a plate boundary. One can simulate either a single-couple
c  or double couple dislocation. 
c  The displacement or stress can be observed at any depth just above the
c  upper locking depth.
c
c  EXAMPLE:
c  An example of a fault extending from a great
c  depth to 30 km depth follows:
c
c******************************************************************
c  fftfault -100000 -30 0 all_segs.dat 0 xall.grd yall.grd zall.grd
c******************************************************************
c
c  The three components of surface displacement are stored
c  in three grd-files for use with GMT software.
c
c  Fault elements are stored in the file all_segs.dat
c
c******************************************************************
c  # dx  sig  lam      mu  (0-shallow 1-deep) (0-single 1-double)
c     1. 1.  3.95e10 4.12e10  0 0
c  549.36 549.36   0.00 128.53 -40.00   0.00   0.00
c  549.36 549.62 128.53 132.51 -40.00   0.00   0.00
c  549.62 549.88 132.51 136.48 -40.00   0.00   0.00
c  549.88 550.26 136.48 142.45 -40.00   0.00   0.00
c  550.26 551.43 142.45 149.49 -40.00   0.00   0.00
c  551.43 552.47 149.49 154.55 -40.00   0.00   0.00
c******************************************************************
c
c  The first line is a comment line.  
c  The second line has; grid spacing(km), fault thickness(km), elastic 
c  constants, shallow/deep flag, single/double-couple flag.
c  Subsequent lines have the end points of the fault followed by slip
c  rates; strike-slip, dip-slip, and opening. Millions of elements
c  could be used without a change in memory or execution time.
c
c  If one specifies deep slip, then a single-couple model is used and a mirror fault is
c  constructed to balance the moment. For an infinitely-long, straight fault,
c  this program generates the arctangent model. 
c
c  The shallow, double-couple model gives the same results at the Okada formula.
c  
c  The only advantages of this program over the standard Okada formulation
c  is that the FFT approach may be orders of magnitude faster.  In addidtion,
c  if you understand the program it is easy to control the forces and
c  boundary conditions.
c
c*****************   main program   ************************
c
      implicit complex (c)
      real kx,ky
      real*8 rln0,rlt0,ddx,ddy,rland,rdum
c
c  change ni and nj as needed
c
      parameter(ni=2048,nj=2048,nwork=32768,nj2=nj/2+1,ni2=ni/2+1)
      character*80 felement,fxdisp,fydisp,fzdisp
      character*80 cd1,cd2,cz,cstr
c
      common/elastic/rlam,rmu,rc,rd,alph,pi
c
      dimension fx(nj,ni),fy(nj,ni),fz(nj,ni)
      dimension u(nj,ni),v(nj,ni),w(nj,ni)
      complex fkx(nj2,ni),fky(nj2,ni),fkz(nj2,ni)
      complex uk(nj2,ni),vk(nj2,ni),wk(nj2,ni),Tk
      dimension n(2)
      complex work(nwork)
      equivalence (fx(1,1),fkx(1,1))
      equivalence (fy(1,1),fky(1,1))
      equivalence (fz(1,1),fkz(1,1))
      equivalence (u(1,1),uk(1,1))
      equivalence (v(1,1),vk(1,1))
      equivalence (w(1,1),wk(1,1))
c
      pi=acos(-1.)
c
c  zero the arrays fx,fy,fz
c
      do 30 i=1,ni
      do 30 j=1,nj
      fx(j,i)=0.
      fy(j,i)=0.
      fz(j,i)=0.
  30  continue
c
c  set the dimensions for fourt
c
      n(1)=nj
      n(2)=ni
c
c   get values from command line
c
      narg = iargc()
      if(narg.lt.8) then
        write(*,'(a)')'  '
        write(*,'(a)')
     &  'Usage: fftfault D1 D2 Zobs ele.dat istr xd.grd yd.grd zd.grd'
        write(*,'(a)')
        write(*,'(a)')
     &  '       D1, D2  - depth to bottom and top of fault (neg) '
        write(*,'(a)')
     &  '       Zobs    - depth of observation plane '
        write(*,'(a)')
     &  '       ele.dat - file of: x1,x2,y1,y2,F1,F2,F3'
        write(*,'(a)')
     &  '       istress - (0)-disp. U,V,W or (1)-stress Txx,Tyy,Txy'
        write(*,'(a)')
     &  '       x,y,z.grd - output files of disp. or stress '
        write(*,'(a)')'  '
        stop
      else 
        call getarg(1,cd1)
        call getarg(2,cd2)
        call getarg(3,cz)
        call getarg(5,cstr)
        read(cd1,*)d1
        read(cd2,*)d2
        read(cz,*)zobs
        read(cstr,*)istr
        if(d2.gt.0..or.d1.ge.d2.or.zobs.lt.d2.or.zobs.gt.0) then
         write(*,'(a)')' depths should be negative and d1 < d2 < zobs'
         stop
        endif
        call getarg(4,felement)
        call getarg(6,fxdisp)
        nc=index(fxdisp,' ')
        fxdisp(nc:nc)='\0'
        call getarg(7,fydisp)
        nc=index(fydisp,' ')
        fydisp(nc:nc)='\0'
        call getarg(8,fzdisp)
        nc=index(fzdisp,' ')
        fzdisp(nc:nc)='\0'
      endif
c
c  open the input file and load the force arrays
c
      open(unit=5,file=felement,status='old')
      read(5,*)cd1
c
c  read the source constants
c
      read(5,*) dx,sig,rlam,rmu,ideep,idble
c
c  remove the double couple for deep sources
c
      if(ideep.eq.1)idble=0
      width=dx*nj
      height=dx*ni
      alph=(rlam+rmu)/(rlam+2*rmu)
      rc=alph/(4.*rmu)
      rd=(rlam+3*rmu)/(rlam+rmu)
      sig=sig/dx
      if(sig.lt.1)sig=1
c
c  read the elements
c
   50 read(5,*,end=999)x1,x2,y1,y2,F1,F2,F3
      x1=x1/dx
      x2=x2/dx
c
c  invert the y-positions to allign with i-index
c
      y1=ni-y1/dx
      y2=ni-y2/dx
c
c
c  figure out the part of the array to add the new forces
c
      ipd=3*sig+2
      jx0=min(x1,x2)-ipd
      jxf=max(x1,x2)+.5+ipd
      iy0=min(y1,y2)-ipd
      iyf=max(y1,y2)+.5+ipd
      do 100 iy=iy0,iyf
      do 200 jx=jx0,jxf
        if(jx.le.0.or.jx.gt.nj) go to 200
        if(iy.le.0.or.iy.gt.ni) go to 200
        x=jx
        y=iy
        call element(x,y,x1,y1,x2,y2,dx,sig,idble,F1,F2,F3,fxx,fyy,fzz)
        fx(jx,iy)=fx(jx,iy)+fxx
        fy(jx,iy)=fy(jx,iy)+fyy
        fz(jx,iy)=fz(jx,iy)+fzz
c
c remove the x-forces if the fault goes deep
c
        if(ideep.eq.1)fx(jx,iy)=0.
 200  continue
 100  continue
      go to 50
 999  close(unit=5)
c
c  if this is a deep source then mirror the source in the x-direction
c  this will make the zero-slope but finite displacement BC's match
c  one could also use a cosine transform
c
      if(ideep.eq.1) then
        do 150 iy=1,ni
        do 150 jx=1,nj2
        fx(nj+1-jx,iy)=-fx(jx,iy)
        fy(nj+1-jx,iy)= fy(jx,iy)
        fz(nj+1-jx,iy)= fz(jx,iy)
 150    continue
      endif
c
c  take the fourier transform of the forces
c
      call fourt(fx,n,2,-1,0,work,nwork)
      call fourt(fy,n,2,-1,0,work,nwork)
      call fourt(fz,n,2,-1,0,work,nwork)
c
c  construct the fault solution
c
      do 255 i=1,ni
      ky=-(i-1)/height
      if(i.ge.ni2) ky= (ni-i+1)/height
      do 255 j=1,nj2
      kx=(j-1)/width
      call fault(kx,ky,d1,d2,zobs,cux,cuy,cuz,
     +                            cvx,cvy,cvz,
     +                            cwx,cwy,cwz,
     +                            c3x,c3y,c3z)
      uk(j,i)=(fkx(j,i)*cux+fky(j,i)*cuy+fkz(j,i)*cuz)/(ni*nj)
      vk(j,i)=(fkx(j,i)*cvx+fky(j,i)*cvy+fkz(j,i)*cvz)/(ni*nj)
      wk(j,i)=(fkx(j,i)*cwx+fky(j,i)*cwy+fkz(j,i)*cwz)/(ni*nj)
      Tk     =(fkx(j,i)*c3x+fky(j,i)*c3y+fkz(j,i)*c3z)/(ni*nj)
c
c  do the boussinesq problem
c
      call boussinesq(kx,ky,zobs,cub,cvb,cwb,cdwdz)
c
c  now either output displacement or stress
c
      uk(j,i)=rmu*(uk(j,i)+Tk*cub)
      vk(j,i)=rmu*(vk(j,i)+Tk*cvb)
      wk(j,i)=rmu*(wk(j,i)+Tk*cwb)
c
c  now compute the stress if requested. 
c
      if(istr.eq.1) then
        cux=cmplx(0.,2.*pi*kx)*uk(j,i)
        cvy=cmplx(0.,2.*pi*ky)*vk(j,i)
        cuy=cmplx(0.,2.*pi*ky)*uk(j,i)
        cvx=cmplx(0.,2.*pi*kx)*vk(j,i)
        cwz=rlam*(cux+cvy)/(rlam+2.*rmu)
c
c  assume units are km so divide by 1000
c
        uk(j,i)=.001*((rlam+2*rmu)*cux+rlam*(cvy+cwz))
        vk(j,i)=.001*((rlam+2*rmu)*cvy+rlam*(cux+cwz))
        wk(j,i)=.001*(rmu*(cuy+cvx))
      endif
 255  continue
c
c  do the inverse fft's
c
      call fourt(u,n,2,1,-1,work,nwork)
      call fourt(v,n,2,1,-1,work,nwork)
      call fourt(w,n,2,1,-1,work,nwork)
c
c  write 3 grd files some parameters must be real*8
c
      rln0=0.
      rlt0=0.
      ddx=dx
      ddy=dx
      rland=9998.
      rdum=9999.
      if(istr.eq.1) then
       rland=9998.d0*rmu
       rdum=9999.d0*rmu
      endif
      call writegrd(u,nj,ni,rln0,rlt0,ddx,ddy,
     +              rland,rdum,fxdisp,fxdisp)
      call writegrd(v,nj,ni,rln0,rlt0,ddx,ddy,
     +              rland,rdum,fydisp,fydisp)
      call writegrd(w,nj,ni,rln0,rlt0,ddx,ddy,
     +              rland,rdum,fzdisp,fzdisp)
      stop
      end
